//+------------------------------------------------------------------+
//|                                                 MarketHelper.mq4 |
//|                                      Copyright  2012, MandySoft |
//|                             http://www.2p.cz/en/forex_metatrader |
//+------------------------------------------------------------------+
#property copyright "Copyright  2012, MandySoft"
#property link      "http://www.2p.cz/en/forex_metatrader"

#property library

#include <MarketDef.mqh>
#include <TZDBHelper.mqh>

string markets[NUM_MARKETS][7] = {
//name, timezone, start, end, currency, avoid weekend, avoid news
  "New York", "America/New_York", "08:00", "17:00", "USD", "1", "1",
  "London", "Europe/London",      "08:00", "17:00", "GBP", "1", "1",
  "Tokyo", "Asia/Tokyo",          "08:00", "17:00", "JPY", "1", "1",
  "Sydney", "Australia/Sydney",   "08:00", "17:00", "AUD", "1", "1",
  "Hong Kong", "Asia/Hong_Kong",  "08:00", "17:00", "CNY", "1", "0",
  "Auckland", "Pacific/Auckland", "08:00", "17:00", "NZD", "1", "0",
  "NY 7-9", "America/New_York",   "07:00", "09:15", "USD", "1", "1"
};

string GetMarketHelperInfo(int marketId, int property) {
  return (markets[marketId][property]);
}

datetime newsDT[];
int newsMask[];

// at least 3 fields  yyyy.m.d;hh:mm;currency sorted by first two fields ascending
#define NEWS_FILENAME "news2.csv"
#define NEWS_TIMEZONE "America/New_York"

bool newsInitialized = false;

void initializeNews() {
  int i;
  if (newsInitialized || NEWS_FILENAME == "") return;
   int h = FileOpen(NEWS_FILENAME, FILE_CSV|FILE_READ, ";");
   if (h == -1) {
      Alert(GetLastError());  
   } else {
      datetime lastDT = 0;
      int mask;
      while (!FileIsEnding(h)) {
         string arr[3];
         for (i=0; i<ArraySize(arr); i++) {
            arr[i] = FileReadString(h);
         }
         while (!FileIsLineEnding(h)) {
            FileReadString(h);
         }
         if (arr[0] == "") continue;
         datetime DT = StrToTime(arr[0]+" "+arr[1]);
         if (DT != lastDT) {
            if (lastDT != 0 && mask != 0) {
              ArrayResize(newsDT, ArraySize(newsDT)+1);
              ArrayResize(newsMask, ArraySize(newsMask)+1);
              newsDT[ArraySize(newsDT)-1] = lastDT;
              newsMask[ArraySize(newsMask)-1] = mask;                                  
            }
            mask = 0;
            lastDT = DT;
         }
         for (i = 0; i<NUM_MARKETS; i++) {
           if (markets[i][MARKET_AVOID_NEWSDAY] == "1") {
             if (arr[2] == "ALL" ||
               markets[i][MARKET_CALENDAR_CURRENCY] == arr[2]) {
               mask |= (1 << i);      
             }
           }
         }
      }
      if (lastDT != 0 && mask != 0) {
         ArrayResize(newsDT, ArraySize(newsDT)+1);
         ArrayResize(newsMask, ArraySize(newsMask)+1);
         newsDT[ArraySize(newsDT)-1] = lastDT;
         newsMask[ArraySize(newsMask)-1] = mask;                                  
      }
      FileClose(h);
   }
   newsInitialized = true;
}

// optinized algorithm, search is executed from previous succesfull search position
// DT is NEWS_TIMEZONE
int getNewsMask(datetime DT1, datetime DT2) {
   static int lastIdx = 0;
   int idx, mask;
   mask = 0;
   int sz = ArraySize(newsDT);
   if (sz == 0) return (mask);
   idx = lastIdx;
   if (DT1 >= newsDT[idx]) {
//Alert("#1:", TimeToStr(DT1) ,">=", TimeToStr(newsDT[idx]));
     while (idx < sz) {
//Alert("#11:", TimeToStr(DT1) ,"<=", TimeToStr(newsDT[idx]));
       if (DT1 <= newsDT[idx]) {
          lastIdx = idx;
          while (idx < sz) {
//Alert("#13:", idx ,"<=", sz);
//Alert("#14:", TimeToStr(DT2) ,"<", TimeToStr(newsDT[idx]));
            if (DT2 < newsDT[idx]) break;
//Alert("#15:", newsMask[idx]);
            mask |= newsMask[idx];
            idx++;
          }
          return(mask);      
       }
       idx++;
     }
   } else if (DT2 <= newsDT[idx]) {
//Alert("#2:", TimeToStr(DT2) ,"<=", TimeToStr(newsDT[idx]));
     while (idx >= 0) {
       if (DT2 >= newsDT[idx]) {
          lastIdx = idx;
          while (idx >= 0) {
            if (DT1 > newsDT[idx]) break;
            mask |= newsMask[idx];
            idx--;           
          }
          return(mask);      
       }
       idx--;
     }
   } else {
     while (idx < sz) {
       if (DT2 < newsDT[idx]) break;
       mask |= newsMask[idx];
       idx++;
     }
     while (idx >= 0) {
       if (DT1 > newsDT[idx]) break;
        mask |= newsMask[idx];
        idx--;
     }
   }
   return (mask);
}   
   


bool IsMarketTime(int marketId, datetime serverDT) {
  double sec1 = 1.0/24.0/60.0/60.0;
 
  if (markets[marketId][MARKET_TZ_IDX] == "") return (false);
  double local = TZLocalToLocalTime(SERVER_TZ, markets[marketId][MARKET_TZ_IDX], TZUnixToDateTime(serverDT), false);   // we do not consider night time shift shadow interval when local time may be ambiguous
         
  double dt1 = TZMidnightTime(local) + TZSinceMidnightTime(TZUnixToDateTime(StrToTime(markets[marketId][MARKET_OPEN_IDX])));
  double dt2 = TZMidnightTime(local) + TZSinceMidnightTime(TZUnixToDateTime(StrToTime(markets[marketId][MARKET_CLOSE_IDX])));
  
  if (local <= (dt1-sec1) || local > (dt2-sec1)) {
    return (false);
  }
  if (markets[marketId][MARKET_AVOID_WEEKEND] == "1") {
    datetime u1 = TZDateTimeToUnix(dt1);
    int DOW = TimeDayOfWeek(u1);
    if (DOW == 0 || DOW == 6) {
      return (false);
    }            
  }
  
  if (markets[marketId][MARKET_AVOID_NEWSDAY] == "1") {
    initializeNews();
    if (ArraySize(newsDT) > 0) {
      double ndt1 = dt1;
      double ndt2 = dt2;
            
      if (markets[marketId][MARKET_TZ_IDX] != NEWS_TIMEZONE) {
        ndt1 = TZToLocalTime(NEWS_TIMEZONE, TZToUniversalTime(markets[marketId][MARKET_TZ_IDX], ndt1, false));
        ndt2 = TZToLocalTime(NEWS_TIMEZONE, TZToUniversalTime(markets[marketId][MARKET_TZ_IDX], ndt2, false));
      } else {
        ndt1 = dt1;
        ndt2 = dt2;
      }
      // whole day events are tickes at 0:01
      int midnight = TZDateTimeToUnix(TZMidnightTime(ndt1));
      
      if ((getNewsMask(midnight, midnight+60) & (1<<marketId)) != 0) {
        return (false);  // news/holiday during session, skip it
      } else if ((getNewsMask(TZDateTimeToUnix(ndt1), TZDateTimeToUnix(ndt1)) & (1<<marketId)) != 0) {
        return (false);
      }         
    }
  }
  return (true);
}

